defmodule Prymn.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    children = [
      # Start the Telemetry supervisor
      PrymnWeb.Telemetry,
      # Start the Ecto repository
      Prymn.Repo,
      # Start the PubSub system
      {DNSCluster, query: Application.get_env(:prymn, :dns_cluster_query) || :ignore},
      {Phoenix.PubSub, name: Prymn.PubSub},
      # Start the Finch HTTP client for sending emails
      {Finch, name: Prymn.Finch},
      # Start the Agents (dynamic GRPC connections) supervisor
      Prymn.Agents.Supervisor,
      # Start a worker by calling: SampleApp.Worker.start_link(arg)
      # {SampleApp.Worker, arg},
      # Start to serve requests, typically the last entry
      PrymnWeb.Endpoint
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: Prymn.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    PrymnWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
